/*************************************************************************************************
** Project:     FSMGenerator
** Package:     FSMApi
** Module:      FSMApi::Cpp
** File:        AbstractFSM.h
** Date:        2002.08.01
** Author:      Pavel Bekkerman (chpavel@tx.technion.ac.il)
** Copyright:   Copyright (C) 2002 Pavel Bekkerman
** License:
**              This program is free software; you can redistribute it and/or
**              modify it under the terms of the GNU General Public License as
**              published by the Free Software Foundation; either version 2 of
**              the License, or (at your option) any later version.
**
**              This program is distributed in the hope that it will be useful,
**              but WITHOUT ANY WARRANTY; without even the implied warranty of
**              MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**              GNU General Public License for more details.
**
**              You should have received a copy of the GNU General Public License
**              along with this program; if not, write to the Free Software
**              Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
**
** Contact:     Pavel Bekkerman (chpavel@tx.technion.ac.il)
*************************************************************************************************/

#ifndef _ABSTRACTFSM_H
#define _ABSTRACTFSM_H

#include "FSMIDPair.h"

namespace FSMApi
{

	// API
	class AbstractFSM
	{
	public:
		virtual int _getStartState() throw() =0; // throws FSMException
		virtual FSMIDPair _getNextIncarnation( int idState, int idEvent ) throw() =0; // throws FSMException
		virtual bool _isFinalState	   ( int idState ) throw() =0; // throws FSMException
		virtual void _processStateEnter  ( int isState ) throw() =0; // throws FSMException
		virtual void _processStateExit   ( int isState ) throw() =0; // throws FSMException
		virtual void _processEvent       ( int idEvent ) throw() =0; // throws FSMException
		virtual void _processCallback    ( int idCallback ) throw() =0; // throws FSMException
		virtual void _processTransition  ( int idTransition ) throw() =0; // throws FSMException
};

} // namespace FSMGenerator

#endif // _ABSTRACTFSM_H
