/*************************************************************************************************
** Project:     FSMGenerator
** Package:     FSMGenerator
** Module:      FSMGenerator::generator
** File:        FSMCGenerator.cpp
** Date:        2002.08.01
** Author:      Pavel Bekkerman (chpavel@tx.technion.ac.il)
** Copyright:   Copyright (C) 2002 Pavel Bekkerman
** License:
**              This program is free software; you can redistribute it and/or
**              modify it under the terms of the GNU General Public License as
**              published by the Free Software Foundation; either version 2 of
**              the License, or (at your option) any later version.
**
**              This program is distributed in the hope that it will be useful,
**              but WITHOUT ANY WARRANTY; without even the implied warranty of
**              MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**              GNU General Public License for more details.
**
**              You should have received a copy of the GNU General Public License
**              along with this program; if not, write to the Free Software
**              Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
**
** Contact:     Pavel Bekkerman (chpavel@tx.technion.ac.il)
*************************************************************************************************/

#include "FSMCGenerator.h"

using namespace std;

const string STATE_FILE_NAME = "IDStates";
const string EVENT_FILE_NAME = "IDEvents";
const string TRANS_FILE_NAME = "IDTransitions";
const string CALLS_FILE_NAME = "IDCallbacks";

const string C_FILE_EXT    = ".c";
const string HEADER_FILE_EXT = ".h";

#define    FSM_C_FILE_NAME        (m_fsmDb.Name() + C_FILE_EXT)
#define    FSM_H_FILE_NAME        (m_fsmDb.Name() + HEADER_FILE_EXT)

// ********************************************************************* FSMCGenerator

// *********************************************** Constructor
FSMGenerator::FSMCGenerator::FSMCGenerator(const FSMDB& fsmDb, const string& strPath)
			: FSMFileGenerator(fsmDb,strPath)
{
}

// *********************************************** Destructor

FSMGenerator::FSMCGenerator::~FSMCGenerator()
{
}

// *********************************************** File Generators

// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ _stateFileGenerator

bool
FSMGenerator::FSMCGenerator::_stateFileGenerator()const
{
	const string strFileName = m_strDestinationPath + STATE_FILE_NAME + HEADER_FILE_EXT;

	ofstream file(strFileName.c_str(),ios::trunc);
	if(!file)
		return false;

	string strDefine = m_fsmDb.Name();

	for(int c=0; c < strDefine.size(); ++c)
		strDefine[c] = toupper(strDefine[c]);

	file<<"#ifndef _"<<strDefine<<"STATES_"<<endl;
	file<<"#define _"<<strDefine<<"STATES_"<<endl<<endl<<endl;


	file 	<< "/*****************************************" << endl
		<< "* Title: IDStates Interface               " << endl
		<< "* Description: State Enumerator interface." << endl
		<< "*****************************************/" << endl
		<< endl
		<< "// created from configuration file"		<< endl
		<< endl;


	file << "typedef enum" << endl
		<< "\t\t{" << endl;

	file<<"\t\t\t"<<STATE_PRE<<",\t\t// the NULL State."<<endl;	

	int state;
	for( state = 0; state < m_fsmDb.m_dvStates.size()-1; ++state )	
		file<<"\t\t\t"<<STATE_PRE<<m_fsmDb.m_dvStates[state].Name()<<','<<endl;

	file << "\t\t\t" << STATE_PRE << m_fsmDb.m_dvStates[state].Name() << endl;


	file<<"\n\t\t} IDStates;\n\n"<<endl;


	file<<"\n\n\n#endif\t//"<<"#ifndef _"<<strDefine<<"STATES_"<<endl;

	return true;
}

// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ _eventFileGenerator

bool
FSMGenerator::FSMCGenerator::_eventFileGenerator()const
{
	const string strFileName = m_strDestinationPath + EVENT_FILE_NAME + HEADER_FILE_EXT;

	ofstream file(strFileName.c_str(),ios::trunc);
	if(!file)
		return false;

	string strDefine = m_fsmDb.Name();

	for(int c=0; c < strDefine.size(); ++c)
		strDefine[c] = toupper(strDefine[c]);

	file<<"#ifndef _"<<strDefine<<"EVENTS_"<<endl;
	file<<"#define _"<<strDefine<<"EVENTS_"<<endl<<endl<<endl;

	file<<"/*****************************************"	<<endl
		<<"* Title: IDEvents Interface"					<<endl
		<<"* Description: Event Enumerator interface."	<<endl
		<<"******************************************/"	<<endl
		<<endl
		<<"// created from configuration file"<<endl
		<<endl;


	file<<"typedef enum "<<endl
		<<"\t\t{"<<endl;

	file<<"\t\t\t"<<EVENT_PRE<<",\t\t// the NULL Event."<<endl;	

	int event;
	for( event = 0; event < m_fsmDb.m_dvEvents.size()-1; ++event )
		file<<"\t\t\t"<<EVENT_PRE<<m_fsmDb.m_dvEvents[event].Name()<<','<<endl;

	file<<"\t\t\t"<<EVENT_PRE<<m_fsmDb.m_dvEvents[event].Name()<<endl;


	file<<"\n\t\t} IDEvents;\n\n"<<endl;

	file<<"\n\n\n#endif\t//"<<"#ifndef _"<<strDefine<<"EVENTS_"<<endl;

	return true;
}

// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ _transFileGenerator

bool
FSMGenerator::FSMCGenerator::_transFileGenerator()const
{
	const string strFileName = m_strDestinationPath + TRANS_FILE_NAME + HEADER_FILE_EXT;

	ofstream file(strFileName.c_str(),ios::trunc);
	if(!file)
		return false;

	string strDefine = m_fsmDb.Name();

	for(int c=0; c < strDefine.size(); ++c)
		strDefine[c] = toupper(strDefine[c]);

	file<<"#ifndef _"<<strDefine<<"TRANSITIONS_"<<endl;
	file<<"#define _"<<strDefine<<"TRANSITIONS_"<<endl<<endl<<endl;


	file<<"/**********************************************"	<<endl
		<<"* Title: IDTransitions Interface"				<<endl
		<<"* Description: Transition Enumerator interface."	<<endl
		<<"***********************************************/"<<endl
		<<endl
		<<"// created from configuration file"<<endl
		<<endl;

	file<<"typedef enum"<<endl
		<<"\t\t{"<<endl;

	file<<"\t\t\t"<<TRANS_PRE<<",\t\t// the NULL Transition."<<endl;	

	int trans;
	for( trans = 0; trans < m_fsmDb.m_dvTransitions.size()-1; ++trans )
		file<<"\t\t\t"<<TRANS_PRE<<m_fsmDb.m_dvTransitions[trans].Name()<<','<<endl;

	file<<"\t\t\t"<<TRANS_PRE<<m_fsmDb.m_dvTransitions[trans].Name()<<endl;


	file<<"\n\t\t} IDTransitions;\n\n"<<endl;

	file<<"\n\n\n#endif\t//"<<"#ifndef _"<<strDefine<<"TRANSITIONS_"<<endl;

	return true;
}

// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ _callsFileGenerator

bool
FSMGenerator::FSMCGenerator::_callsFileGenerator()const
{
	const string strFileName = m_strDestinationPath + CALLS_FILE_NAME + HEADER_FILE_EXT;

	ofstream file(strFileName.c_str(),ios::trunc);
	if(!file)
		return false;

	string strDefine = m_fsmDb.Name();

	for(int c=0; c < strDefine.size(); ++c)
		strDefine[c] = toupper(strDefine[c]);

	file<<"#ifndef _"<<strDefine<<"CALLBACKS_"<<endl;
	file<<"#define _"<<strDefine<<"CALLBACKS_"<<endl<<endl<<endl;

	file<<"/********************************************"	<<endl
		<<"* Title: IDCallbacks Interface"					<<endl
		<<"* Description: Callback Enumerator interface."	<<endl
		<<"*********************************************/"	<<endl
		<<endl
		<<"// created from configuration file"<<endl
		<<endl;

	file<<"typedef enum"<<endl
		<<"\t\t{"<<endl;

	file<<"\t\t\t"<<CALL_PRE<<",\t\t// the NULL Callback."<<endl;	

	int call;
	for( call = 1; call < m_fsmDb.m_dvCallbacks.size()-1; ++call )
		file<<"\t\t\t"<<CALL_PRE<<m_fsmDb.m_dvCallbacks[call].Name()<<','<<endl;

	file<<"\t\t\t"<<CALL_PRE<<m_fsmDb.m_dvCallbacks[call].Name()<<endl;


	file<<"\n\t\t} IDCallbacks;\n\n"<<endl;

	file<<"\n\n\n#endif\t//"<<"#ifndef _"<<strDefine<<"CALLBACKS_"<<endl;

	return true;
}

// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ _mainFileGenerator

bool
FSMGenerator::FSMCGenerator::_mainFileGenerator()const
{
	////////////////////////////////////////////
	// header generation.

	const string strHeaderFileName = m_strDestinationPath+FSM_H_FILE_NAME;

	ofstream file(strHeaderFileName.c_str(),ios::trunc);
	if(!file)
		return false;

	string strDefine = m_fsmDb.Name();

	for(int c=0; c < strDefine.size(); ++c)
		strDefine[c] = toupper(strDefine[c]);

	file<<"#ifndef _"<<strDefine<<'_'<<endl;
	file<<"#define _"<<strDefine<<'_'<<endl<<endl<<endl;

	file<<"/******************************************"	<<endl
		<<"* Title: class "<<m_fsmDb.Name()				<<endl
		<<"* Description: The main class of the FSM"	<<endl
		<<"*******************************************/"<<endl
		<<endl
		<<"// created from configuration file"<<endl
		<<endl;

	file<<"#include \"IDPair.h\""<<endl;
	file<<"#include \""<<STATE_FILE_NAME + HEADER_FILE_EXT<<'\"'<<endl;
	file<<"#include \""<<EVENT_FILE_NAME + HEADER_FILE_EXT<<'\"'<<endl;
	file<<"#include \""<<TRANS_FILE_NAME + HEADER_FILE_EXT<<'\"'<<endl;
	file<<"#include \""<<CALLS_FILE_NAME + HEADER_FILE_EXT<<'\"'<<endl;
	file<<endl<<endl;

	// function declarations
	file<<"\t\tIDStates\t_getStartState\t\t();"<<endl
		<<"\t\tIDPair\t_getNextIncarnation\t( IDStates idState, IDEvents idEvent );"<<endl
		<<"\t\tbool\t_isFinalState\t\t( IDStates idState );"<<endl
		<<"\t\tvoid\t_processStateEnter\t( IDStates idState );"<<endl
		<<"\t\tvoid\t_processStateExit\t( IDStates idState );"<<endl
		<<"\t\tvoid\t_processEvent\t\t( IDEvents idEvent );"<<endl
		<<"\t\tvoid\t_processTransition\t( IDTransitions idTransition );"<<endl
		<<"\t\tvoid\t_processCallback\t( IDCallbacks idCallback );"<<endl;

	file<<"\n};\t //class "<<m_fsmDb.Name()<<endl;

	// end of class declaration

	file<<"\n\n\n#endif\t//"<<"#ifndef _"<<strDefine<<'_'<<endl;

	file.close();

	// end of header generation .

	////////////////////////////////////////////
	// cpp generation.

	const string strCPPFileName = m_strDestinationPath+FSM_C_FILE_NAME;
	file.open(strCPPFileName.c_str(),ios::trunc);

	file<<"#include \""<<FSM_H_FILE_NAME<<'\"'<<endl<<endl;

	file<<"// ****************************************** class "<<m_fsmDb.Name()<<" functions.\n"<<endl;
	// generate all member functions.

	_mainGetStartStateGenerator		(file);
	file<<endl;
	_mainGetNextIncarnationGenerator(file);
	file<<endl;
	_mainIsFinalStateGenerator		(file);
	file<<endl;
	_mainProcessStateEnterGenerator	(file);
	file<<endl;
	_mainProcessStateExitGenerator	(file);
	file<<endl;
	_mainProcessEventGenerator		(file);
	file<<endl;
	_mainProcessTransitionGenerator	(file);
	file<<endl;
	_mainProcessCallbackGenerator	(file);
	file<<endl;

	// end of FSM class.
	return true;
}

// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ _mainGetStartState

void
FSMGenerator::FSMCGenerator::_mainGetStartStateGenerator(ofstream& file)const
{
	file<<"// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ _getStartState\n"<<endl;

	file<<"inline IDStates\n"
		<<m_fsmDb.Name()<<"::"<<m_fsmDb.Name()<<"::_getStartState()\n"
		<<"{\n"
		<<"\treturn STATE_"<<m_fsmDb.m_dvStates[m_fsmDb.m_indStart].Name()<<";\n"
		<<"}\n";	
}

// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ _mainGetNextIncarnation

void
FSMGenerator::FSMCGenerator::_mainGetNextIncarnationGenerator(ofstream& file)const
{
	file<<"// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ _getNextIncarnation\n"<<endl;

	file<<"IDPair\n"
		<<m_fsmDb.Name()<<"::"<<m_fsmDb.Name()<<"::_getNextIncarnation( IDStates idState, IDEvents idEvent )\n"
		  "{\n"
		  "\tIDPair idpair;\n\n"
		  "\tswitch( idState )\n"
		  "\t{"<<endl;

	int state;
	for( state = 0; state<m_fsmDb.m_dvStates.size(); ++state )
	{
		const State& current = m_fsmDb.m_dvStates[state];
		file<<"\t\tcase STATE_"<<current.Name()<<":\n";
		file<<"\t\t\tswitch( idEvent )\n"
			  "\t\t\t{"<<endl;
		list<INDEX>::const_iterator citr;
		for(citr = current.transitions.begin(); citr != current.transitions.end(); ++citr)
		{
			const Transition& trans = m_fsmDb.m_dvTransitions[*citr];
			file<<"\t\t\t\tcase EVENT_"<<m_fsmDb.m_dvEvents[trans.WithEvent()].Name()<<":\n"
				  "\t\t\t\t\tidpair.idFirst  = TRANSITION_"<<trans.Name()<<";\n"
				  "\t\t\t\t\tidpair.idSecond = STATE_"<<m_fsmDb.m_dvStates[trans.ToState()].Name()<<";\n"
			      "\t\t\t\tbreak;\n"<<endl;
		}

		file<<"\t\t\t\tdefault:\n"
              "\t\t\t\t\tthrow new FSMException( \"Illegal event.\" );\n"
			  "\t\t\t}\n"
			  "\t\tbreak;\n"<<endl;
	}

	file<<"\t\tdefault:\n"
          "\t\t\tthrow new FSMException( \"Illegal state.\" );\n"
		  "\t}\n"<<endl;

	file<<"\t return idpair;\n"
		  "}\n";
}

// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ _mainIsFinalStateGenerator

void
FSMGenerator::FSMCGenerator::_mainIsFinalStateGenerator(ofstream& file)const
{
	file<<"// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ _isFinalState\n"<<endl;

	file<<"bool\n"
		<<m_fsmDb.Name()<<"::"<<m_fsmDb.Name()<<"::_isFinalState( IDStates idState )\n"
		"{\n"
		"\tswitch( idState )\n"
		"\t{\n";

	int state;
	for( state=0; state < m_fsmDb.m_dvStates.size(); ++state )
		if(m_fsmDb.m_dvStates[state].IsFinal())
			file<<"\t\tcase STATE_"<<m_fsmDb.m_dvStates[state].Name()<<":\n";

	file<<"\t\t\treturn true;\n"<<endl;

	for( state=0; state < m_fsmDb.m_dvStates.size(); ++state )
		if(!m_fsmDb.m_dvStates[state].IsFinal())
			file<<"\t\tcase STATE_"<<m_fsmDb.m_dvStates[state].Name()<<":\n";
	file<<"\t\t\treturn false;\n"<<endl;

	file<<"\t\tdefault:\n"
		  "\t\t\tthrow new FSMException( \"Illegal state.\" );\n";
	file<<"\t}\n"
		  "}\n";
}

// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ _mainProcessStateEnterGenerator

void
FSMGenerator::FSMCGenerator::_mainProcessStateEnterGenerator(ofstream& file)const
{
	file<<"// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ _processStateEnter\n"<<endl;

	file<<"void\n"
		<<m_fsmDb.Name()<<"::"<<m_fsmDb.Name()<<"::_processStateEnter( IDStates idState )\n"
		  "{\n"
		  "\ttry\n"
		  "\t{\n"
		  "\t\tswitch( idState )\n"
		  "\t\t{\n";

	int state;
	for( state=0; state < m_fsmDb.m_dvStates.size(); ++state )
	{
		const State& current = m_fsmDb.m_dvStates[state];
		file<<"\t\t\tcase STATE_"<<current.Name()<<":\n"
			  "\t\t\t\t_processCallback( CALLBACK_"<<m_fsmDb.m_dvCallbacks[current.EnterCall()].Name()<<" );\n"
			  "\t\t\tbreak;\n"<<endl;
	}
    file<<"\t\t\tdefault:\n"
		  "\t\t\t\tthrow new FSMException( \"Illegal state.\" );\n"
		  "\t\t}\n"
		  "\t}\n"
		  "\tcatch( FSMException e )\n"
		  "\t{\n"
		  "\t\tthrow e;\n"
		  "\t}\n"
		  "}\n";
}

// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ _mainProcessStateExitGenerator

void
FSMGenerator::FSMCGenerator::_mainProcessStateExitGenerator(ofstream& file)const
{
	file<<"// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ _processStateExit\n"<<endl;

	file<<"void\n"
		<<m_fsmDb.Name()<<"::"<<m_fsmDb.Name()<<"::_processStateExit( IDStates idState )\n"
		  "{\n"
		  "\ttry\n"
		  "\t{\n"
		  "\t\tswitch( idState )\n"
		  "\t\t{\n";

	int state;
	for( state=0; state < m_fsmDb.m_dvStates.size(); ++state )
	{
		const State& current = m_fsmDb.m_dvStates[state];
		file<<"\t\t\tcase STATE_"<<current.Name()<<":\n"
			  "\t\t\t\t_processCallback( CALLBACK_"<<m_fsmDb.m_dvCallbacks[current.ExitCall()].Name()<<" );\n"
			  "\t\t\tbreak;\n"<<endl;
	}
    file<<"\t\t\tdefault:\n"
		  "\t\t\t\tthrow new FSMException( \"Illegal state.\" );\n"
		  "\t\t}\n"
		  "\t}\n"
		  "\tcatch( FSMException e )\n"
		  "\t{\n"
		  "\t\tthrow e;\n"
		  "\t}\n"
		  "}\n";
}


// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ _mainProcessEventGenerator

void
FSMGenerator::FSMCGenerator::_mainProcessEventGenerator(ofstream& file)const
{
	file<<"// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ _processEvent\n"<<endl;

	file<<"void\n"
		<<m_fsmDb.Name()<<"::"<<m_fsmDb.Name()<<"::_processEvent( IDEvents idEvent )\n"
		  "{\n"
		  "\ttry\n"
		  "\t{\n"
		  "\t\tswitch( idEvent )\n"
		  "\t\t{\n";

	int event;
	for( event=0; event < m_fsmDb.m_dvEvents.size(); ++event )
	{
		const Event& current = m_fsmDb.m_dvEvents[event];
		file<<"\t\t\tcase EVENT_"<<current.Name()<<":\n"
			  "\t\t\t\t_processCallback( CALLBACK_"<<m_fsmDb.m_dvCallbacks[current.Callback()].Name()<<" );\n"
			  "\t\t\tbreak;\n"<<endl;
	}
    file<<"\t\t\tdefault:\n"
		  "\t\t\t\tthrow new FSMException( \"Illegal event.\" );\n"
		  "\t\t}\n"
		  "\t}\n"
		  "\tcatch( FSMException e )\n"
		  "\t{\n"
		  "\t\tthrow e;\n"
		  "\t}\n"
		  "}\n";
}


// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ _mainProcessTransitionGenerator

void
FSMGenerator::FSMCGenerator::_mainProcessTransitionGenerator(ofstream& file)const
{
	file<<"// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ _processTransition\n"<<endl;

	file<<"void\n"
		<<m_fsmDb.Name()<<"::"<<m_fsmDb.Name()<<"::_processTransition( IDTransitions idTransition )\n"
		  "{\n"
		  "\ttry\n"
		  "\t{\n"
		  "\t\tswitch( idTransition )\n"
		  "\t\t{\n";

	int trans;
	for( trans=0; trans < m_fsmDb.m_dvTransitions.size(); ++trans )
	{
		const Transition& current = m_fsmDb.m_dvTransitions[trans];
		file<<"\t\t\tcase TRANSITION_"<<current.Name()<<":\n"
			  "\t\t\t\t_processCallback( CALLBACK_"<<m_fsmDb.m_dvCallbacks[current.Callback()].Name()<<" );\n"
			  "\t\t\tbreak;\n"<<endl;
	}
    file<<"\t\t\tdefault:\n"
		  "\t\t\t\tthrow new FSMException( \"Illegal transition.\" );\n"
		  "\t\t}\n"
		  "\t}\n"
		  "\tcatch( FSMException e )\n"
		  "\t{\n"
		  "\t\tthrow e;\n"
		  "\t}\n"
		  "}\n";
}

// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ _mainProcessCallbackGenerator

void
FSMGenerator::FSMCGenerator::_mainProcessCallbackGenerator(ofstream& file)const
{
	file<<"// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ _processCallback\n"<<endl;

	file<<"void\n"
		<<m_fsmDb.Name()<<"::"<<m_fsmDb.Name()<<"::_processCallback( IDCallbacks idCallback )\n"
		  "{\n"
		  "\tswitch( idCallback )\n"
		  "\t{\n";

	// add the case for the 'empty' callback.
	file<<"\t\tcase CALLBACK_:\n"
		  "\t\t\t// The 'NULL' callback - nothing to be done.\n"
		  "\t\tbreak;\n"<<endl;

	int call;
	for( call=1; call < m_fsmDb.m_dvCallbacks.size(); ++call ) // skip the 1st 'empty'.
	{
		const Callback& current = m_fsmDb.m_dvCallbacks[call];
		file<<"\t\tcase CALLBACK_"<<current.Name()<<":\n"
			  "\t\t\t"<<current.Function()<<"();\n"
			  "\t\tbreak;\n"<<endl;
	}

	file<<"\t\tdefault:\n"
          "\t\t\tthrow new FSMException( \"Illegal callback.\" );\n"
		  "\t}\n"
		  "\treturn;\n"
		  "}\n";
}

// ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ _addStaticFiles

bool
FSMGenerator::FSMCGenerator::_addStaticFiles()const
{
	return true;
}

