/*************************************************************************************************
** Project:     FSMGenerator
** Package:     FSMGenerator
** Module:      FSMGenerator::generator
** File:        FSMFileGenerator.cpp
** Date:        2002.08.01
** Author:      Pavel Bekkerman (chpavel@tx.technion.ac.il)
** Copyright:   Copyright (C) 2002 Pavel Bekkerman
** License:
**              This program is free software; you can redistribute it and/or
**              modify it under the terms of the GNU General Public License as
**              published by the Free Software Foundation; either version 2 of
**              the License, or (at your option) any later version.
**
**              This program is distributed in the hope that it will be useful,
**              but WITHOUT ANY WARRANTY; without even the implied warranty of
**              MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**              GNU General Public License for more details.
**
**              You should have received a copy of the GNU General Public License
**              along with this program; if not, write to the Free Software
**              Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
**
** Contact:     Pavel Bekkerman (chpavel@tx.technion.ac.il)
*************************************************************************************************/

#include "FSMFileGenerator.h"

using namespace std;

#ifdef WIN32
	#include <direct.h>
#else
	#include <sys/stat.h>
	#include <sys/types.h>
#endif

#include <stdlib.h>
#include <errno.h>

// ********************************************************************* FSMFileGenerator

// *********************************************** Constructor
FSMGenerator::FSMFileGenerator::FSMFileGenerator(const FSMDB& fsmDb, const string& strPath)
			: m_fsmDb(fsmDb), m_strDestinationPath(strPath)
{
	m_strDestinationPath += "/" + m_fsmDb.Name() + "/";
}

// *********************************************** Destructor

FSMGenerator::FSMFileGenerator::~FSMFileGenerator()
{
}

// *********************************************** GenerateFiles
int
FSMGenerator::FSMFileGenerator::_directoryCreate()
{
#ifdef WIN32
	return _mkdir( m_strDestinationPath.c_str() );
#else
	return mkdir( m_strDestinationPath.c_str(), S_IRWXU );
#endif
}
// *********************************************** GenerateFiles

bool
FSMGenerator::FSMFileGenerator::GenerateFiles()
{
	if(_directoryCreate())
	{
		if( errno != EEXIST )	// if it's not because the directory is already exists.
			return false;
	}

	return	_stateFileGenerator() &&
			_eventFileGenerator() &&
			_transFileGenerator() &&
			_callsFileGenerator() &&
			_mainFileGenerator ();
}
